////////////////////////////////////////////////////////////////////
/*
  Env_senseP1.c
  December 14, 2013 
*/
#include "simpletools.h"
#include "simpletext.h"
#include "fdserial.h"

#include "sRTC.h"
#include "sht11.h"

////////////////////////////////////////////////////////////////////
/* Auto sd_mount of SD. */
extern _Driver _NullDriver;
extern _Driver _FileDriver;

_Driver *_driverlist[] = {
  &_NullDriver,
  &_FileDriver,
  NULL
};
////////////////////////////////////////////////////////////////////
serial *xbee;
FILE *sp;
////////////////////////////////////////////////////////////////////
static volatile float sht11_temp;
static volatile float sht11_humid;
static volatile int logstat;  // Control senselog.
static volatile int sleeptime;  // Control in seconds.
static volatile int runtime;  // Control in hours.


unsigned int sht11stack[40 + 150];  // 40 + 25
unsigned int sensestack[40 + 300];  // 40 + 150,40 + 25

////////////////////////////////////////////////////////////////////
void menu(void);
void get_Time(void);
void get_TD(void);
void set_Time(void);
void set_Clock(void);
int prompttime(char *prompt);
/* Write to SD. */
void date_SD(void);
void time_SD(void);

/* COG functions. */
void sht11(void* par);
void senselog(void* par);


////////////////////////////////////////////////////////////////////
int main()
{
  // Add startup code here.
/* Open a file for append. */
  if((sp=fopen("SenseD.txt","a"))==0)
  {
    while(1)
    {
      high(27);    // Red LED, error on open.
      pause(500);
      low(27);
      pause(500);
    }
  }
  else
  {
    high(26);  // Green LED, file was opened.
    fclose(sp);
  }

/* Start the sht11 cog. */
  cogstart(&sht11, NULL, sht11stack, sizeof(sht11stack));
/* Start the senselog cog. */
  cogstart(&senselog, NULL, sensestack, sizeof(sensestack));
  pause(100);

/* Init the XBee module. */
/*                     Rx  Tx mode BAUD */
  xbee = fdserial_open(25, 24, 0, 9600);
  pause(50);
/* Start the Sensirion RTC. */
  sRTC_Start();
  sRTC_Update();
  sRTC_Set24hourmode(1);
  sRTC_Update();
  char inBuff[40];

  menu();
 
  while(1)
  {
    // Add main loop code here.
    writeStr(xbee,"> ");
    readStr(xbee, inBuff, 40);
    if(!strcmp(inBuff, "help")) menu();
    else if(!strcmp(inBuff, "time")) get_Time();
    else if(!strcmp(inBuff, "date")) get_TD();
    else if(!strcmp(inBuff, "temp"))
    {
      writeFloat(xbee,sht11_temp);
      writeStr(xbee,"\n");
    }
    else if(!strcmp(inBuff, "humid"))
    {
      writeFloat(xbee,sht11_humid);
      writeStr(xbee,"\n");
    }
    else if(!strcmp(inBuff, "settime"))
    {
      writeStr(xbee,"Hour,Minute,Second\n");
      set_Time();
    }
    else if(!strcmp(inBuff, "setclock"))
    {
      writeStr(xbee,"Month,Day,Year xx,dow,Hour,Minutes,Seconds\n");
      writeStr(xbee,"dow: 0-Sun,1-Mon,2-Tue,3-Wed,4-Thu,5-Fri,6-Sat\n");
      set_Clock();
    }
    else if(!strcmp(inBuff,"five24")) //Every 5 minutes for 24 hours
    {
      sleeptime = 300;  // 5 minutes (60 secs x 5 min = 300)
      runtime = 288;    // 24 hours  (12 counts x 24 hours = 288)
      logstat = 1;
    }
    else if(!strcmp(inBuff,"five1")) // Every 5 minutes for 1 hour
    {
      sleeptime = 300;  // 5 minutes (60 secs x 5 min = 300)
      runtime = 12;     // 1 hour  (12 counts x 1 hour = 12)
      logstat = 1;
    }
    else
    {
      writeLine(xbee,"Invalid Command!");
    }    
  }  
}
////////////////////////////////////////////////////////////////////

void menu()
{
  writeStr(xbee, "Menu - help, five24, five1, time, date, \n");
  writeStr(xbee, "        temp, humid, settime, setclock, \n");
}

////////////////////////////////////////////////////////////////////
void sht11(void* par)
{
/*           data clk */
  sht11_pins(17,  16);

  while(1)
  {
/* Raw temp */
    int soT = sht11_tempRaw();
    //print("soT=%d", soT);
/* Raw real humidity */
    int soRH = sht11_rhRaw();
    //print(" soRH=%d", soRH);
/* Degrees C */
    float sTC = sht11_degC();
    //print(" sTC=%f",sTC);
/* Degrees F */
    float sTf = sht11_degF();
    sht11_temp = sTf;
    //print(" sTf=%f", sTf);
//    sprint(outBuff,"%f",sTf);
//    fwrite(outBuff,1,8,sp);
//    fwrite("F ",1,2,sp);    
/* Raw humidity Linear */
    float sRHL = sht11_rhLinear();
   // print(" sRHL=%f", sRHL);
/* Raw humidity True */
    float sRHT = sht11_rhTrue();
    sht11_humid = sRHT;
    //print(" sRHT=%f\n", sRHT);
//    sprint(outBuff,"%f",sRHT);
//    fwrite(outBuff,1,8,sp);
//    fwrite("H\r\n",1,3,sp); 
/* Dew point C */
    float sDPC = sht11_dewPointC(); 
    //print(" sDPC=%f\n", sDPC);
    pause(200);
  }
}

////////////////////////////////////////////////////////////////////
void senselog(void* par)
{
  int sostat;
  int timer = 1;
  int stop;
  while(1)
  {
    char outBuff[] = {' ',' ',' ',' ',' ',' ',' ',' ',};
    sostat = logstat;
   
    if(sostat == 1)
    {
      high(27);
      sp = fopen("SenseD.txt", "a");
      fwrite("\r\n",1,2,sp);  // CR
      date_SD();             // Get date.
      fwrite("\r\n",1,2,sp);  // CR
      while(1)
      {
        //if(stoplog == 0) break;  // Test for external stop command.
        time_SD();             // Get time.
        fwrite("  ",1,2,sp);
        sprint(outBuff,"%f",sht11_temp);
        fwrite(outBuff,1,8,sp);  // Write temp.
        fwrite("  ",1,2,sp);
        sprint(outBuff,"%f",sht11_humid);
        fwrite(outBuff,1,8,sp);  // Write humid.
        fwrite("\r\n",1,2,sp);  // CR
        sleep(sleeptime);   // Sleep for five minutes. Sleep time
        stop = timer++;
        if(stop == runtime) break; // 24 hours. Runtime
      }    
      fclose(sp);  // Close the file.
      low(27);     // Turn visual que off.
      logstat = 0; 
    }
    pause(250);
  }
}

////////////////////////////////////////////////////////////////////
/* Write date to SD */
void date_SD(void)
{
  char outBuff[] = {' ',' '};
  int day, month, year;
  sRTC_Update();
  month = sRTC_Getmonth();
  day = sRTC_Getday();
  year = sRTC_Getyear();
  sprint(outBuff,"%d",month);
  fwrite(outBuff,1,2,sp);
  fwrite("/",1,1,sp);
  sprint(outBuff,"%d",day);
  fwrite(outBuff,1,2,sp);
  fwrite("/",1,1,sp);
  sprint(outBuff,"%d",year);
  fwrite(outBuff,1,2,sp);
}

////////////////////////////////////////////////////////////////////
void time_SD(void)
/* Write time to SD.  */
{
  char outBuff[] = {' ',' '};
  int hour, minute, second;
  sRTC_Update();
  hour = sRTC_Gethour();
  minute = sRTC_Getminutes();
  second = sRTC_Getseconds();
  sprint(outBuff,"%d",hour);
  fwrite(outBuff,1,2,sp);
  fwrite(":",1,1,sp);
  sprint(outBuff,"%d",minute);
  fwrite(outBuff,1,2,sp);
  fwrite(":",1,1,sp);
  sprint(outBuff,"%d",second);
  fwrite(outBuff,1,2,sp);
}

////////////////////////////////////////////////////////////////////
void get_Time(void)
{
  int result, result1, result2;
  sRTC_Update();
  result = sRTC_Gethour();
  result1 = sRTC_Getminutes();
  result2 = sRTC_Getseconds();
  writeDecDigits(xbee,result,2);
  writeStr(xbee,":");
  writeDecDigits(xbee,result1,2);
  writeStr(xbee,":");
  writeDecDigits(xbee,result2,2);
  writeStr(xbee,"\n");
}

////////////////////////////////////////////////////////////////////
void get_TD(void)
{
  int result, result1, result2;
  sRTC_Update();
  result = sRTC_Getmonth();
  result1 = sRTC_Getday();
  result2 = sRTC_Getyear();
  writeDecDigits(xbee,result,2);
  writeStr(xbee,"/");
  writeDecDigits(xbee,result1,2);
  writeStr(xbee,"/");
  writeDecDigits(xbee,result2,2);
  writeStr(xbee,"\n");
}

////////////////////////////////////////////////////////////////////


int prompttime(char *prompt)
{
  int rc = 0;
  char *endp;
  char buffer[10];

  do {
      writeStr(xbee,"Enter ");
      writeStr(xbee,prompt);
      writeStr(xbee," ");

      readStr(xbee,buffer,10);
      rc = strtol(buffer, &endp, 10);
      if(endp == buffer)
      {
        if('\n' == *endp)
        {
          rc = 0;
          break;
        }
        writeStr(xbee,"Invalid entry");
        writeChar(xbee,*endp);
        writeStr(xbee,"....");
        writeStr(xbee," Please enter a number.\n");
      }
  } while(endp == 10);

  return rc;
}

////////////////////////////////////////////////////////////////////
void set_Time()
{
  int hour,minute,second;
  hour = prompttime("Hour");
  minute = prompttime("Minute");
  second = prompttime("Second");
  writeDec(xbee,hour);
  writeStr(xbee,":");
  writeDec(xbee,minute);
  writeStr(xbee,":");
  writeDec(xbee,second);
  sRTC_Settime(hour,minute,second);
  sRTC_Update();
  writeStr(xbee,"\n");
}

////////////////////////////////////////////////////////////////////
void set_Clock()
{
  int month,day,year,dow,hour,minute,second;
  month = prompttime("Month");
  day = prompttime("Day");
  year = prompttime("Year");
  dow = prompttime("DayOfWeek");
  hour = prompttime("Hour");
  minute = prompttime("Minute");
  second = prompttime("Second");

  writeStr(xbee,"\n");
  writeDec(xbee,month);
  writeStr(xbee," ");
  writeDec(xbee,day);
  writeStr(xbee," ");
  writeDec(xbee,year);
  writeStr(xbee," ");
  writeDec(xbee,dow);
  writeStr(xbee," ");
  writeDec(xbee,hour);
  writeStr(xbee," ");
  writeDec(xbee,minute);
  writeStr(xbee," ");
  writeDec(xbee,second);
  writeStr(xbee," ");
  writeStr(xbee,"\n");
  sRTC_Setdatetime(month,day,year,dow,hour,minute,second);
  sRTC_Update();  
}


